/* $Id: RegisterDialog.java,v 1.1 2007/10/19 07:29:57 nito Exp $
 *
 * Copyright (c)ARGO 21, Corporation. 2005, 2006.  All rights reserved.
 * 
 * This file is part of Nautica Workflow.
 * 
 *  Nautica Workflow is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Nautica Workflow is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Nautica Workflow; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *  
 */
package jp.co.argo21.nautica.tool.wfadm.definition.register;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import jp.co.argo21.nautica.commons.util.ResourceManager;
import jp.co.argo21.nautica.tool.wfadm.WorkflowAdminManager;
import jp.co.argo21.nautica.tool.wfadm.common.DialogComponent;

/**
 * 定義ファイル登録情報入力用のダイアログを提供する。
 * 
 * @author  nito(Argo 21, Corp.)
 * @version $Revision: 1.1 $
 * @since   Nautica Workflow 1.0
 */
public class RegisterDialog extends JDialog
{
	/** シリアルバージョンID */
	private static final long serialVersionUID = 1L;

	/** 表示中のDialogComponent */
	private DialogComponent dialogComponent;
	/** 検証が通ったかどうかのフラグ */
	private boolean commited;

	/**
	 * ダイアログを表示する。
	 * 
	 * @param c	親コンポーネント
	 * @param elem	要素
	 * @return	更新された要素
	 */
	public static boolean showDialog(Component c, DialogComponent dc)
	{
		Frame frame = JOptionPane.getFrameForComponent(c);
		RegisterDialog dialog = new RegisterDialog(frame, dc);
		dialog.setVisible(true);
		boolean committed = dialog.commited;
		dialog.dispose();

		return committed;
	}

	/**
	 * ダイアログを生成する。
	 * 
	 * @param frame 親フレーム
	 * @param dc ダイアログコンポーネント
	 */
	private RegisterDialog(Frame frame, DialogComponent dc)
	{
		super(frame, true);
		
		dialogComponent = dc;
		createDialog();
		setLocationRelativeTo(frame);
	}

	/**
	 * ダイアログを作成する。
	 */
	private void createDialog()
	{
		ResourceManager rm = WorkflowAdminManager.getResourceManager();

		setTitle(rm.getResource("RegisterDialog.title"));
		
		JScrollPane scroll = new JScrollPane();
		scroll.setViewportView((JComponent)dialogComponent);

		JPanel buttonPane = new JPanel();
		buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT, 10, 10));

		ActionListener listener = new CloseAction();
		JButton setButton = new JButton(rm.getResource("JButton.label.set"));
		setButton.setActionCommand("OK_OPTION");
		setButton.addActionListener(listener);
		getRootPane().setDefaultButton(setButton);
		buttonPane.add(setButton);

		JButton cancelButton = new JButton(rm.getResource("JButton.label.cancel"));
		cancelButton.addActionListener(listener);
		buttonPane.add(cancelButton);

		Container contentPane = getContentPane();
		contentPane.add(scroll, BorderLayout.CENTER);
		contentPane.add(buttonPane, BorderLayout.PAGE_END);

		pack();
	}

	/**
	 * ダイアログが閉じたときの処理
	 */
	private class CloseAction implements ActionListener
	{
		/**
		 * 検証を行ってから閉じる。
		 * 
		 * @param e	イベント
		 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
		 */
		public void actionPerformed(ActionEvent e)
		{
			if ("OK_OPTION".equals(e.getActionCommand())) {
				if (dialogComponent.commit() == false) return;
				commited = true;
			}
			setVisible(false);
		}
	}
}
